@@ -22,14 +22,14 @@ def expr(val, nesting_depth=20):
 
     if isinstance(val, RqlQuery):
         return val
-    elif isinstance(val, datetime.datetime):
-        if not val.tzinfo:
-            raise RqlDriverError("""Cannot convert datetime to ReQL time object
+    elif isinstance(val, datetime.datetime) or isinstance(val, datetime.date):
+        if not hasattr(val, 'tzinfo') or not val.tzinfo:
+            raise RqlDriverError("""Cannot convert %s to ReQL time object
             without timezone information. You can add timezone information with
             the third party module \"pytz\" or by constructing ReQL compatible
             timezone values with r.make_timezone(\"[+-]HH:MM\"). Alternatively,
             use one of ReQL's bultin time constructors, r.now, r.time, or r.iso8601.
-            """)
+            """ % (type(val).__name__))
         return ISO8601(val.isoformat())
     elif isinstance(val, list):
         val = [expr(v, nesting_depth - 1) for v in val]
@@ -313,6 +313,12 @@ class RqlQuery(object):
         elif isinstance(index, types.StringTypes):
             return GetField(self, index)
 
+    def __iter__(self):
+        raise RqlDriverError(
+                "__iter__ called on an RqlQuery object.\n"+
+                "To iterate over the results of a query, call run first.\n"+
+                "To iterate inside a query, use map or for_each.")
+
     def nth(self, index):
         return Nth(self, index)
 
@@ -492,18 +498,24 @@ class RqlTzinfo(datetime.tzinfo):
         self.offsetstr = offsetstr
         self.delta = datetime.timedelta(hours=hours, minutes=minutes)
 
+    def __copy__(self):
+        return RqlTzinfo(self.offsetstr)
+
+    def __deepcopy__(self, memo):
+        return RqlTzinfo(self.offsetstr)
+
     def utcoffset(self, dt):
         return self.delta
 
     def tzname(self, dt):
-        return offsetstr
+        return self.offsetstr
 
     def dst(self, dt):
         return datetime.timedelta(0)
 
 def reql_type_time_to_datetime(obj):
     if not 'epoch_time' in obj:
-        raise RqlDriverError('psudo-type TIME object %s does not have expected field "epoch_time".' % py_json.dumps(obj))
+        raise RqlDriverError('pseudo-type TIME object %s does not have expected field "epoch_time".' % py_json.dumps(obj))
 
     if 'timezone' in obj:
         return datetime.datetime.fromtimestamp(obj['epoch_time'], RqlTzinfo(obj['timezone']))
@@ -546,35 +558,21 @@ class Datum(RqlQuery):
 
     @staticmethod
     def deconstruct(datum, time_format='native'):
-        if datum.type == p.Datum.R_NULL:
-            return None
-        elif datum.type == p.Datum.R_BOOL:
-            return datum.r_bool
-        elif datum.type == p.Datum.R_NUM:
-            # Convert to an integer if we think maybe the user might think of this
-            # number as an integer. I have been assured that this is a "temporary"
-            # behavior change until RQL supports native integers.
-            num = datum.r_num
-            if num % 1 == 0:
-                # Then we assume that in the user's data model this floating point
-                # number is meant be an integer and "helpfully" convert types for them.
-                num = int(num)
-            return num
-        elif datum.type == p.Datum.R_STR:
-            return datum.r_str
-        elif datum.type == p.Datum.R_ARRAY:
-            return [Datum.deconstruct(e, time_format) for e in datum.r_array]
-        elif datum.type == p.Datum.R_OBJECT:
-            obj = {}
+        d_type = datum.type
+        if d_type == p.Datum.R_JSON:
+            return py_json.loads(datum.r_str)
+        elif d_type == p.Datum.R_OBJECT:
+            obj = { }
             for pair in datum.r_object:
                 obj[pair.key] = Datum.deconstruct(pair.val, time_format)
 
-            # Thanks to "psudo-types" we can't yet be quite sure if this object is meant to
+            # Thanks to "pseudo-types" we can't yet be quite sure if this object is meant to
             # be an object or something else. We need a second layer of type switching, this
             # time on an obfuscated field "$reql_type$" rather than the datum type field we
             # already switched on.
-            if '$reql_type$' in obj:
-                if obj['$reql_type$'] == 'TIME':
+            reql_type = obj.get('$reql_type$')
+            if reql_type is not None:
+                if reql_type == 'TIME':
                     if time_format == 'native':
                         # Convert to native python datetime object
                         return reql_type_time_to_datetime(obj)
@@ -584,9 +582,28 @@ class Datum(RqlQuery):
                     else:
                         raise RqlDriverError("Unknown time_format run option \"%s\"." % time_format)
                 else:
-                    raise RqlDriverError("Unknown psudo-type %" % obj['$reql_type$'])
+                    raise RqlDriverError("Unknown pseudo-type %s" % reql_type)
 
             return obj
+        elif d_type == p.Datum.R_ARRAY:
+            array = datum.r_array
+            return [Datum.deconstruct(e, time_format) for e in array]
+        elif d_type == p.Datum.R_STR:
+            return datum.r_str
+        elif d_type == p.Datum.R_NUM:
+            # Convert to an integer if we think maybe the user might think of this
+            # number as an integer. I have been assured that this is a "temporary"
+            # behavior change until RQL supports native integers.
+            num = datum.r_num
+            if num % 1 == 0:
+                # Then we assume that in the user's data model this floating point
+                # number is meant be an integer and "helpfully" convert types for them.
+                num = int(num)
+            return num
+        elif d_type == p.Datum.R_BOOL:
+            return datum.r_bool
+        elif d_type == p.Datum.R_NULL:
+            return None
         else:
             raise RuntimeError("Unknown Datum type %d encountered in response." % datum.type)
 
@@ -826,6 +843,15 @@ class Table(RqlQuery):
     def index_list(self):
         return IndexList(self)
 
+    def index_status(self, *indexes):
+        return IndexStatus(self, *indexes)
+
+    def index_wait(self, *indexes):
+        return IndexWait(self, *indexes)
+
+    def sync(self):
+        return Sync(self)
+
     def compose(self, args, optargs):
         if isinstance(self.args[0], DB):
             return T(args[0], '.table(', args[1], ')')
@@ -890,10 +916,6 @@ class IsEmpty(RqlMethodQuery):
     tt = p.Term.IS_EMPTY
     st = 'is_empty'
 
-class IndexesOf(RqlMethodQuery):
-    tt = p.Term.INDEXES_OF
-    st = 'indexes_of'
-
 class GroupedMapReduce(RqlMethodQuery):
     tt = p.Term.GROUPED_MAP_REDUCE
     st = 'grouped_map_reduce'
@@ -990,6 +1012,18 @@ class IndexList(RqlMethodQuery):
     tt = p.Term.INDEX_LIST
     st = 'index_list'
 
+class IndexStatus(RqlMethodQuery):
+    tt = p.Term.INDEX_STATUS
+    st = 'index_status'
+
+class IndexWait(RqlMethodQuery):
+    tt = p.Term.INDEX_WAIT
+    st = 'index_wait'
+
+class Sync(RqlMethodQuery):
+    tt = p.Term.SYNC
+    st = 'sync'
+
 class Branch(RqlTopLevelQuery):
     tt = p.Term.BRANCH
     st = "branch"
